MODULE tranpc
   !!==============================================================================
   !!                       ***  MODULE  tranpc  ***
   !! Ocean active tracers:  non penetrative convection scheme
   !!==============================================================================
   !! History :  1.0  ! 1990-09  (G. Madec)  Original code
   !!                 ! 1996-01  (G. Madec)  statement function for e3
   !!   NEMO     1.0  ! 2002-06  (G. Madec)  free form F90
   !!            3.0  ! 2008-06  (G. Madec)  applied on ta, sa and called before tranxt in step.F90
   !!----------------------------------------------------------------------

   !!----------------------------------------------------------------------
   !!   tra_npc : apply the non penetrative convection scheme
   !!----------------------------------------------------------------------
   USE oce             ! ocean dynamics and active tracers 
   USE dom_oce         ! ocean space and time domain
   USE zdf_oce         ! ocean vertical physics
   USE trdmod          ! ocean active tracer trends
   USE trdmod_oce      ! ocean variables trends
   USE eosbn2          ! equation of state (eos routine) 
   USE lbclnk          ! lateral boundary conditions (or mpp link)
   USE in_out_manager  ! I/O manager

   IMPLICIT NONE
   PRIVATE

   PUBLIC   tra_npc    ! routine called by step.F90

   !! * Substitutions
#  include "domzgr_substitute.h90"
   !!----------------------------------------------------------------------
   !! NEMO/OPA 3.2 , LOCEAN-IPSL (2009) 
   !! $Id$ 
   !! Software governed by the CeCILL licence (modipsl/doc/NEMO_CeCILL.txt)
   !!----------------------------------------------------------------------

CONTAINS

   SUBROUTINE tra_npc( kt )
      !!----------------------------------------------------------------------
      !!                  ***  ROUTINE tranpc  ***
      !!
      !! ** Purpose :   Non penetrative convective adjustment scheme. solve 
      !!      the static instability of the water column on after fields
      !!      while conserving heat and salt contents.
      !!
      !! ** Method  :   The algorithm used converges in a maximium of jpk 
      !!      iterations. instabilities are treated when the vertical density
      !!      gradient is less than 1.e-5.
      !!      l_trdtra=T: the trend associated with this algorithm is saved.
      !!
      !! ** Action  : - (ta,sa) after the application od the npc scheme
      !!              - save the associated trends (ttrd,strd) ('key_trdtra')
      !!
      !! References : Madec, et al., 1991, JPO, 21, 9, 1349-1371.
      !!----------------------------------------------------------------------
      USE oce, ONLY :    ztrdt => ua   ! use ua as 3D workspace   
      USE oce, ONLY :    ztrds => va   ! use va as 3D workspace   
      !! 
      INTEGER, INTENT(in) ::   kt   ! ocean time-step index
      !!
      INTEGER  ::   ji, jj, jk   ! dummy loop indices
      INTEGER  ::   inpcc        ! number of statically instable water column
      INTEGER  ::   inpci        ! number of iteration for npc scheme
      INTEGER  ::   jiter, jkdown, jkp        ! ???
      INTEGER  ::   ikbot, ik, ikup, ikdown   ! ???
      REAL(wp) ::   ze3tot, zta, zsa, zraua, ze3dwn
      REAL(wp), DIMENSION(jpi,jpk)     ::   zwx, zwy, zwz   ! 2D arrays
      REAL(wp), DIMENSION(jpi,jpj,jpk) ::   zrhop           ! 3D arrays
      !!----------------------------------------------------------------------

      IF( MOD( kt, nn_npc ) == 0 ) THEN

         inpcc = 0
         inpci = 0

         CALL eos( ta, sa, rhd, zrhop )         ! Potential density


         IF( l_trdtra )   THEN                  ! Save ta and sa trends
            ztrdt(:,:,:) = ta(:,:,:) 
            ztrds(:,:,:) = sa(:,:,:) 
         ENDIF

         !                                                ! ===============
         DO jj = 1, jpj                                   !  Vertical slab
            !                                             ! ===============
            !  Static instability pointer 
            ! ----------------------------
            DO jk = 1, jpkm1
               DO ji = 1, jpi
                  zwx(ji,jk) = ( zrhop(ji,jj,jk) - zrhop(ji,jj,jk+1) ) * tmask(ji,jj,jk+1)
               END DO
            END DO

            ! 1.1 do not consider the boundary points

            ! even if east-west cyclic b. c. do not considere ji=1 or jpi
            DO jk = 1, jpkm1
               zwx( 1 ,jk) = 0.e0
               zwx(jpi,jk) = 0.e0
            END DO
            ! even if south-symmetric b. c. used, do not considere jj=1
            IF( jj == 1 )   zwx(:,:) = 0.e0

            DO jk = 1, jpkm1
               DO ji = 1, jpi
                  zwx(ji,jk) = 1.
                  IF( zwx(ji,jk) < 1.e-5 ) zwx(ji,jk) = 0.e0
               END DO
            END DO

            zwy(:,1) = 0.e0
            DO ji = 1, jpi
               DO jk = 1, jpkm1
                  zwy(ji,1) = zwy(ji,1) + zwx(ji,jk)
               END DO
            END DO

            zwz(1,1) = 0.e0
            DO ji = 1, jpi
               zwz(1,1) = zwz(1,1) + zwy(ji,1)
            END DO

            inpcc = inpcc + NINT( zwz(1,1) )


            ! 2. Vertical mixing for each instable portion of the density profil
            ! ------------------------------------------------------------------

            IF( zwz(1,1) /= 0.e0 ) THEN         ! -->> the density profil is statically instable :
               DO ji = 1, jpi
                  IF( zwy(ji,1) /= 0.e0 ) THEN
                     !
                     ikbot = mbathy(ji,jj)      ! ikbot: ocean bottom level
                     !
                     DO jiter = 1, jpk          ! vertical iteration
                        !
                        ! search of ikup : the first static instability from the sea surface
                        !
                        ik = 0
220                     CONTINUE
                        ik = ik + 1
                        IF( ik >= ikbot-1 ) GO TO 200
                        zwx(ji,ik) = zrhop(ji,jj,ik) - zrhop(ji,jj,ik+1)
                        IF( zwx(ji,ik) <= 0.e0 ) GO TO 220
                        ikup = ik
                        ! the density profil is instable below ikup
                        ! ikdown : bottom of the instable portion of the density profil
                        ! search of ikdown and vertical mixing from ikup to ikdown
                        !
                        ze3tot= fse3t(ji,jj,ikup)
                        zta   = ta   (ji,jj,ikup)
                        zsa   = sa   (ji,jj,ikup)
                        zraua = zrhop(ji,jj,ikup)
                        !
                        DO jkdown = ikup+1, ikbot-1
                           IF( zraua <= zrhop(ji,jj,jkdown) ) THEN
                              ikdown = jkdown
                              GO TO 240
                           ENDIF
                           ze3dwn =  fse3t(ji,jj,jkdown)
                           ze3tot =  ze3tot + ze3dwn
                           zta   = ( zta*(ze3tot-ze3dwn) + ta(ji,jj,jkdown)*ze3dwn )/ze3tot
                           zsa   = ( zsa*(ze3tot-ze3dwn) + sa(ji,jj,jkdown)*ze3dwn )/ze3tot
                           zraua = ( zraua*(ze3tot-ze3dwn) + zrhop(ji,jj,jkdown)*ze3dwn )/ze3tot
                           inpci = inpci+1
                        END DO
                        ikdown = ikbot-1
240                     CONTINUE
                        !
                        DO jkp = ikup, ikdown-1
                           ta(ji,jj,jkp) = zta
                           sa(ji,jj,jkp) = zsa
                           zrhop(ji,jj,jkp) = zraua
                        END DO
                        IF (ikdown == ikbot-1 .AND. zraua >= zrhop(ji,jj,ikdown) ) THEN
                           ta(ji,jj,ikdown) = zta
                           sa(ji,jj,ikdown) = zsa
                           zrhop(ji,jj,ikdown) = zraua
                        ENDIF
                     END DO
                  ENDIF
200               CONTINUE
               END DO
               ! <<-- no more static instability on slab jj
            ENDIF
            !                                             ! ===============
         END DO                                           !   End of slab
         !                                                ! ===============
         ! 
         IF( l_trdtra )   THEN         ! save the Non penetrative mixing trends for diagnostic
            ztrdt(:,:,:) = ta(:,:,:) - ztrdt(:,:,:)
            ztrds(:,:,:) = sa(:,:,:) - ztrds(:,:,:)
            CALL trd_mod(ztrdt, ztrds, jptra_trd_npc, 'TRA', kt)
         ENDIF
      
         ! Lateral boundary conditions on ( ta, sa )   ( Unchanged sign)
         ! ------------------------------============
         CALL lbc_lnk( ta, 'T', 1. )
         CALL lbc_lnk( sa, 'T', 1. )
      

         !  2. non penetrative convective scheme statistics
         !  -----------------------------------------------
         IF( nn_npcp /= 0 .AND. MOD( kt, nn_npcp ) == 0 ) THEN
            IF(lwp) WRITE(numout,*)' kt=',kt, ' number of statically instable',   &
               &                   ' water column : ',inpcc, ' number of iteration : ',inpci
         ENDIF
         !
      ENDIF
      !
   END SUBROUTINE tra_npc

   !!======================================================================
END MODULE tranpc
